# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - ApproveChanges

    Permit the approval of an edited page queued by the page editor when an
    unprivileged user attempted to save a page. Since queued pages are placed in
    a subpage area, this action just moves the queued page content into the
    existing page when approving the changes.

    @copyright: 2011 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

Dependencies = ['pages']

from MoinMoin.action import ActionBase
from MoinMoin.PageEditor import PageEditor
from MoinMoin.wikiutil import escape
from ApproveChangesSupport import *

# Action class and supporting functions.

class ApproveChanges(ActionBase):

    "An action which approves a queued page version."

    def __init__(self, pagename, request):
        ActionBase.__init__(self, pagename, request)
        _ = self._
        self.form_trigger = "approve"
        self.form_trigger_label = _("Approve changes")

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        fmt = request.formatter

        d = {
            "buttons_html"          : buttons_html,
            "prompt"                : escape(_("Approve the displayed page version?")),
            "purge_label"           : escape(_("Purge all other queued versions"))
            }

        # Prepare the output HTML.

        html = '''
<table>
    <tr>
        <td colspan="2">%(prompt)s</td>
    </tr>
    <tr>
        <td class="label"><label>%(purge_label)s</label></td>
        <td>
            <input name="purge" type="checkbox" value="yes" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>''' % d

        return html

    def do_action(self):

        "Approve the page and move it into place."

        _ = self._
        request = self.request

        # Make sure that only suitably privileged users can perform this action.

        queued_changes_area = get_queued_changes_area(request)
        reviewers_group = getattr(request.cfg, "reviewers_group", "PageReviewersGroup")

        if not request.user.valid or (
            not request.dicts.has_member(reviewers_group, request.user.name) and \
            not request.user.isSuperUser()):

            return 0, _("Only page reviewers can perform this action.")

        # Edit the target page, using this page's content.
        # The current page must be a queued page version.

        if not is_queued_page(self.pagename, queued_changes_area):
            return 0, _("This page is not queued for approval.")

        target_page_name = get_target_page_name(self.pagename)
        target_page = PageEditor(request, target_page_name)

        # Save the target page.

        try:
            target_page.saveText(self.page.get_raw_body(), 0)
        except PageEditor.Unchanged:
            pass

        # Delete the current page.

        current_page = PageEditor(request, self.pagename)
        current_page.deletePage(_("Changes to page approved."))

        # Delete the rest of the queue if requested.

        form = get_form(request)

        if form.get("purge"):
            for name in get_queue_pages(request, target_page_name, queued_changes_area):
                queue_page = PageEditor(request, name)
                queue_page.deletePage(_("Changes to page rejected."))

        # Redirect to the target page.

        request.http_redirect(target_page.url(request))
        return 1, None

    def render_success(self, msg, msgtype):

        """
        Render neither 'msg' nor 'msgtype' since redirection should occur
        instead.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

# Action function.

def execute(pagename, request):
    ApproveChanges(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
